
/**
 * Write a description of class Numerik here.
 * 
 * @author (Martin Kell) 
 * @version (0.01)
 */
public class Numerik {

    /**
     * einfacher (brute-force) Primzahlentest
     * 
     * @param  p  eine beliebige positive Zahl
     * @return    ist p eine Primzahl (boolean)
     */
    public boolean isPrimeSimple(int p) {
        if(p < 2) {
            return false;   
        } 
        for(int i = 2; i < p; i++) {
            if(p % i == 0)
                return false;
        }
        return true;    
    }
    
    /**
     * (brute-force) Primzahlentest bis wurzel(p)
     * 
     * @param  p  eine beliebige positive Zahl
     * @return    ist p eine Primzahl (boolean)
     */
    public boolean isPrimeExt(int p) {
        if(p < 2) 
            return false;   
            
        if(p == 3 || p == 2)
            return true;
        if(p % 2 == 0 || p % 3 == 0)
            return false;
            
        // wurzel bilden
        int wurzel = (int) java.lang.Math.sqrt((double)p) + 1;
        
        for(int i = 1; 6*i - 1 < wurzel; i++) {
            if(p % (6*i - 1) == 0)
                return false;
            if(p % (6*i + 1) == 0)
                return false;
        }
        return true;    
    }
    
    /**
     * schnelle Modulare Exponentiation (rekursive):
     *    berechnet a ^ b mod n
     *    
     *    @param  a   Basis (a < n)
     *    @param  b   Exponent (b < n)
     *    @param  n   Modulo (n^2 sollte Integer Bereich nicht berschreiten)
     *    @return     a ^ b mod n
     */
    public int modExp(int a, int b, int n) {
        // bringt a und b in Bereich von n
        a = a % n;
        b = b % n;
        
        if(n*n < 0)
            return -1;
        
        // Rekursionsabbruch und Absicherung
        if(a == 0) // 0^a = 0
            return 0;
        if(a == 1 || b == 0) // 1^a == 1, x^0 = 1
            return 1;
        if(b == 1) // a^1 == a
            return a;
        
        // Halbierungsexponentiation
        if(b % 2 == 0) {
            a = modExp(a, b/2, n);
            return (a*a) % n; 
        } else
            return (a*modExp(a, b-1, n)) % n;
    }
    
    /**
     * Pseudoprimzahlentest mit Fermat (a ^ (p-1) mod p = 1)
     * 
     * @param  p  Primzahl??
     * @param  t  Iterationen (erhht Wahrscheinlichkeit)
     * @return    ist p eine Primzahl (nicht 100%!!)
     */
    boolean isPrimeFermat(int p, int t) {
        for(int i = 0; i < t; i++) {
            int a = (int)(java.lang.Math.random() * (double)p);
            if(modExp(a, p-1, p) != 1)
                return false;
        }
        return true;
    }
    
    /**
     * Primzahlenermittlung (nach einer Zahl n)
     * 
     * @param  n  n+1 ist Beginn der Suche
     * @return    Primzahl
     */
    int getNextPrime(int n) {
        if(n == 1)
            return 2;
        if(n % 2 == 1)
            n++;
        n++;
        
        int p = -1;
        while(true) {
            if(n*n < 0) {
                p = -1;
                break;
            }
            p = n;
            if(isPrimeFermat(n, 4) && isPrimeExt(n))
                break;
            
            n += 2; // immer ungerade Zahlen
            
        }
        return p;
    }
}
